-----------------------------------------------------------------------------------------------
-- Client Lua Script for RaidFrameBase
-- Code here is frop RaidFrameBase.lua but only the relevant part for the Ready Check
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
require "Window"
require "GameLib"
require "GroupLib"
require "ChatSystemLib"

local GroupLib = GroupLib
local Apollo = Apollo
local GameLib = GameLib
local os = os
local math = math
local String_GetWeaselString = String_GetWeaselString
local pairs = pairs
local ChatSystemLib = ChatSystemLib
local setmetatable = setmetatable

local ReadyCheckWithoutRaidFrameBase = {}
local addon = ReadyCheckWithoutRaidFrameBase

function addon:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self
	
	return o
end

function addon:Init()
	Apollo.RegisterAddon(self)
end

local knReadyCheckTimeout = 60 -- in seconds

local knSaveVersion = 3

function addon:OnLoad()
	Apollo.RegisterEventHandler("Group_ReadyCheck",							"OnGroup_ReadyCheck", self)
	Apollo.RegisterTimerHandler("ReadyCheckTimeout", 						"OnReadyCheckTimeout", self)

	if self.strReadyCheckInitiator and self.strReadyCheckMessage then
		local strMessage = String_GetWeaselString(Apollo.GetString("RaidFrame_ReadyCheckStarted"), self.strReadyCheckInitiator) .. "\n" .. self.strReadyCheckMessage
		self.wndReadyCheckPopup = Apollo.LoadForm("ReadyCheckWithoutRaidFrameBase.xml", "RaidReadyCheck", nil, self)

		self.wndReadyCheckPopup:FindChild("ReadyCheckMessage"):SetText(strMessage)
	else
		self.wndReadyCheckPopup 	= nil
	end
end

function addon:OnGroup_ReadyCheck(nMemberIdx, strMessage)
	local tMember = GroupLib.GetGroupMember(nMemberIdx)
	local strName = Apollo.GetString("RaidFrame_TheRaid")
	if tMember then
		strName = tMember.strCharacterName
	end

	if self.wndReadyCheckPopup and self.wndReadyCheckPopup:IsValid() then
		self.wndReadyCheckPopup:Destroy()
	end

	self.wndReadyCheckPopup = Apollo.LoadForm("ReadyCheckWithoutRaidFrameBase.xml", "RaidReadyCheck", nil, self)
	self.wndReadyCheckPopup:FindChild("ReadyCheckMessage"):SetText(String_GetWeaselString(Apollo.GetString("RaidFrame_ReadyCheckStarted"), strName) .. "\n" .. strMessage)

	self.nNumReadyCheckResponses = 0

	self.strReadyCheckInitiator = strName
	self.strReadyCheckMessage = strMessage
	self.fReadyCheckStartTime = os.clock()
	self.bReadyCheckActive = true

	Apollo.CreateTimer("ReadyCheckTimeout", knReadyCheckTimeout, false)
end

function addon:OnReadyCheckResponse(wndHandler, wndControl)
	if wndHandler == wndControl then
		GroupLib.SetReady(wndHandler:GetName() == "ReadyCheckYesBtn") -- TODO Quick Hack
	end

	if self.wndReadyCheckPopup and self.wndReadyCheckPopup:IsValid() then
		self.wndReadyCheckPopup:Destroy()
	end
end

function addon:OnReadyCheckTimeout()
	self.nNumReadyCheckResponses = -1

	if self.wndReadyCheckPopup and self.wndReadyCheckPopup:IsValid() then
		self.wndReadyCheckPopup:Destroy()
	end

	self.bReadyCheckActive = false
end


function addon:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return
	end

	local bHasReadyCheck = false
	if self.wndReadyCheckPopup and self.wndReadyCheckPopup:IsValid() then
		bHasReadyCheck = self.wndReadyCheckPopup:IsShown()
	end

	local tSave =
	{
		bReadyCheckShown 		= bHasReadyCheck,
		fReadyCheckStartTime 	= self.fReadyCheckStartTime,
		strReadyCheckInitiator	= self.strReadyCheckInitiator,
		strReadyCheckMessage	= self.strReadyCheckMessage,
		nReadyCheckResponses 	= self.nNumReadyCheckResponses,
		nSaveVersion 			= knSaveVersion,
	}

	return tSave
end

function addon:OnRestore(eType, tSavedData)
	self.tSavedData = tSavedData
	if not tSavedData or tSavedData.nSaveVersion ~= knSaveVersion then
		return
	end

	local fDelta = tSavedData.fReadyCheckStartTime and os.clock() - tSavedData.fReadyCheckStartTime or knReadyCheckTimeout
	if fDelta < knReadyCheckTimeout then
		self.nNumReadyCheckResponses = 0
		self.fReadyCheckStartTime = tSavedData.fReadyCheckStartTime

		if tSavedData.bReadyCheckShown then
			if self.nNumReadyCheckResponses >= tSavedData.nReadyCheckResponses then

				self.strReadyCheckInitiator = tSavedData.strReadyCheckInitiator
				self.strReadyCheckMessage = tSavedData.strReadyCheckMessage
			end
		end
		Apollo.CreateTimer("ReadyCheckTimeout", math.ceil(knReadyCheckTimeout - fDelta), false)
	end
end

local ReadyCheckWithoutRaidFrameBaseInst = addon:new()
ReadyCheckWithoutRaidFrameBaseInst:Init()